--[[
	Omni Cooldown Count
		A universal cooldown count, based on Gello's spec
--]]

local function PrintMsg(message)
	DEFAULT_CHAT_FRAME:AddMessage(message or "error");
end

--[[ Variable Loading ]]--

local function LoadDefaults(currentVersion)
	OmniCC = {
		min = 3, --minimum duration to show text
		font = STANDARD_TEXT_FONT, --cooldown text font; ClearFont loads before OmniCC and will be used if its there.
		--[[ 추가 ]]--
		font_name = {"2002", "2002B", "ARIALN", "FRIZQT__", "K_Damage", "K_Pagetext"},	-- 와우 기본 내장 폰트 목록
		font_prefix = {"Fonts\\", "Interface\\Addons\\!OmniCC\\Fonts\\"},		-- 기본/추가 폰트 경로
		font_suffix = ".TTF",	-- 폰트 확장자
		--[[ 추가 끝 ]]--
		size = 20, --cooldown text size
		vlong = {r = 0.8, g = 0.8, b = 0.9, s = 0.6}, --settings for cooldowns greater than an hour
		long = {r = 0.8, g = 0.8, b = 0.9, s = 0.8}, --settings for cooldowns greater than one minute
		medium = {r = 1, g = 1, b = 0.4, s = 1}, --settings for cooldowns under a minute
		short = {r = 1, g = 0, b = 0, s = 1.3}, --settings for cooldowns less than five seconds
		shine = nil,
		shineScale = 4,
		version = currentVersion,
	}
	if (GetLocale()=="koKR") then
		PrintMsg("!OmniCC: 기본 설정값을 불려옵니다.");
	else
		PrintMsg("OmniCC: Default Settings Loaded.");
	end
end

local function LoadVariables()
	local version = GetAddOnMetadata("!OmniCC", "Version");
	
	if not(OmniCC and OmniCC.version) then
		LoadDefaults(version);
	elseif OmniCC.version ~= version then
		OmniCC.version = version;
		if (GetLocale()=="koKR") then
			PrintMsg("!OmniCC: 업데이트 버전" .. OmniCC.version);
		else
			PrintMsg("OmniCC: Updated to v" .. OmniCC.version);
		end
	end
	
	--this font is created solely for testing if the user's selection is a valid font or not.
	if not OmniCCFont then
		CreateFont("OmniCCFont");
		if not OmniCCFont:SetFont(OmniCC.font, OmniCC.size) then
			if (GetLocale()=="koKR") then
				PrintMsg("!OmniCC: 폰트를 알수 없습니다. 기본 폰트로 설정합니다.")
			else
				PrintMsg("OmniCC: Saved font is invalid. Reverting to default font.")
			end
			OmniCC.font = STANDARD_TEXT_FONT;
			if not OmniCCFont:SetFont(OmniCC.font, OmniCC.size) then
				OmniCC.font = "Fonts\\K_Damage.TTF";
			end
		end
	end
end

--[[ Slash Command Handler ]]--

local function PrintCommands()
	if (GetLocale()=="koKR") then
		PrintMsg("통합 재사용 대기시간 표시기(!OmniCC) 명령어 :");
		PrintMsg("/옴니 크기 <숫자> - 크기 설정 (현재 설정 : " ..OmniCC.size..")");
		PrintMsg("/옴니 글꼴 <숫자> - 글꼴 설정 (현재 설정 : " ..OmniCC.font..")");
		for k ,v in pairs(OmniCC.font_name) do
			if fontlist then
				fontlist = fontlist .. "  "..k..". "..v;
			else
				fontlist = k..". "..v;
			end
		end
		PrintMsg("  사용 가능한 글꼴 목록 :");
		PrintMsg("  "..fontlist);

		PrintMsg("/옴니 색상 <기간> <적> <녹> <청> - 기간별 색상 설정 (기간은 <vlong>일/시간, <long>분, <medium>초, <short>5초이하 입니다.)");
		PrintMsg("/옴니 비율 <기간> <비율> - 기간별 비율 설정 (기간은 <vlong>일/시간, <long>분, <medium>초, <short>5초이하 이며, 비율에서 원래 크기는 1입니다.)");
		PrintMsg("/옴니 최소 <숫자> - 대기시간 타이머가 감지할 최소 시간 설정 (현재 설정 : "..OmniCC.min.."초)");
		PrintMsg("/옴니 모델 - 기본 블리자드 대기시간 표시 설정");
		PrintMsg("/옴니 점멸 - 대기시간 종료후 플래시 효과 설정");
		PrintMsg("/옴니 점멸크기 <숫자> - 대기시간 종료후 플래시 효과 크기 설정 (현재 설정 : " ..OmniCC.shineScale..")");
	else
		PrintMsg("OmniCC Commands:");
		PrintMsg("/omnicc size <value> - Set font size.  20 is default.");
		PrintMsg("/omnicc font <value> - Set the font to use.  " .. STANDARD_TEXT_FONT .. " is default.");
		PrintMsg("/omnicc color <duration> <red> <green> <blue> - Set the color to use for cooldowns of <duration>.  Duration can be vlong, long, medium or short.");
		PrintMsg("/omnicc scale <duration> <value> - Set the scale to use for cooldowns of <duration>.  Duration can be vlong, long, medium or short.");
		PrintMsg("/omnicc min <value> - Set the minimum duration (seconds) a cooldown should be to show text.  Default value of 3.");
		PrintMsg("/omnicc model - Toggles the cooldown model");
		PrintMsg("/omnicc shine - Toggles a brighter flash on finished cooldowns");
		PrintMsg("/omnicc shinescale <value> - Sets how big the bright cooldown flash is.  4 is default");
		PrintMsg("/omnicc reset - Go back to default settings.");
	end
end

SlashCmdList["OmniCCCOMMAND"] = function(msg)
	if not msg or msg == "" or msg == "help" or msg == "?" or msg == "도움" or msg == "도움말" then
		PrintCommands();
	else
		local args = {};
		for word in string.gmatch(msg, "[^%s]+") do
			table.insert(args, word);
		end
		cmd = string.lower(args[1]);
		
		--/omnicc size <size>
		if (cmd == "size" or cmd == "크기") then
			if(tonumber(args[2]) and tonumber(args[2]) > 0) then
				OmniCC.size = tonumber(args[2]);
				PrintMsg("!OmniCC:글꼴 크기 : "..args[2].." Pt");
			else
				PrintMsg("!OmniCC:글꼴의 크기는 0보다 커야 합니다.");
			end
		--/omnicc font <font>
		elseif (cmd == "font" or cmd == "글꼴" and args[2]) then
			if ( tonumber(args[2]) ) then
				if ( tonumber(args[2]) <= tonumber(getn(OmniCC.font_name)) ) then
					OmniCC.font = OmniCC.font_prefix[1]..OmniCC.font_name[tonumber(args[2])]..OmniCC.font_suffix;
					PrintMsg("!OmniCC:글꼴이 "..OmniCC.font_name[tonumber(args[2])].." 로 변경되었습니다.");
				else
					PrintMsg("!OmniCC:기본 폰트는 숫자로 선택하십시요. 입력 가능한 숫자는 1~"..getn(OmniCC.font_name).." 입니다.");
					PrintMsg("!OmniCC:올바른 글꼴 입력전까지 "..OmniCC.font.." 글꼴을 사용합니다.");
					OmniCC.font = OmniCC.font_prefix[1]..OmniCC.font_name[1]..OmniCC.font_suffix;
				end
			else
				OmniCC.font = OmniCC.font_prefix[2]..args[2]..OmniCC.font_suffix;
				PrintMsg("!OmniCC:글꼴 설정 : "..args[2]);
			end
		--/omnicc min <size>
		elseif (cmd == "min" or cmd == "최소") then
			if tonumber(args[2]) then
				OmniCC.min  = tonumber(args[2]);
				PrintMsg("!OmniCC:최소 감지 시간이 "..tonumber(args[2]).."초로 설정되었습니다.");
			end
		elseif (cmd == "model" or cmd == "모델") then
			if OmniCC.hideModel then
				OmniCC.hideModel = nil;
				PrintMsg("!OmniCC: 자체 재사용 대기시간 기능을 표시합니다.");
			else
				OmniCC.hideModel = 1;
				PrintMsg("!OmniCC: 자체 재사용 대기시간 기능을 표시하지 않습니다.");
			end
		elseif (cmd == "color" or cmd == "색상") then
			if args[2] and tonumber(args[3]) and tonumber(args[4]) and tonumber(args[5]) then
				local index = string.lower(args[2]);
				if index == "vlong" or index == "long" or index == "short" or index == "medium" then
					OmniCC[index].r = tonumber(args[3]);
					OmniCC[index].g = tonumber(args[4]);
					OmniCC[index].b = tonumber(args[5]);
				end
			end
		elseif (cmd == "scale" or cmd == "비율") then
			if args[2] and tonumber(args[3]) then
				local index = string.lower(args[2]);
				if index == "vlong" or index == "long" or index == "short" or index == "medium" then
					OmniCC[index].s = tonumber(args[3]);
				end
			end
		elseif (cmd == "shine" or cmd == "점멸") then
			if OmniCC.shine then
				OmniCC.shine = nil;
				PrintMsg("!OmniCC: 플래시 효과를 표시하지 않습니다.");
			else
				OmniCC.shine = 1;
				PrintMsg("!OmniCC: 플래시 효과를 표시합니다.");
			end
		elseif (cmd == "shinescale" or cmd == "점멸크기") then
			if args[2] and tonumber(args[2]) then
				OmniCC.shineScale = tonumber(args[2]);
				PrintMsg("!OmniCC:플래시 효과 점멸 크기가 "..tonumber(args[2]).."으로 설정되었습니다.");
			end
		elseif (cmd == "reset" or cmd == "리셋" or cmd == "초기화") then
			PrintMsg("!OmniCC:모든 설정값을 기본값으로 초기화합니다.");
			LoadDefaults();
		end
	end
end
SLASH_OmniCCCOMMAND1 = "/omnicc";
SLASH_OmniCCCOMMAND2 = "/재사용";
SLASH_OmniCCCOMMAND3 = "/옴니";
SLASH_OmniCCCOMMAND4 = "/옴";

--[[ Event Handler ]]--

CreateFrame("Frame", "OmniCCMain");
OmniCCMain:SetScript("OnEvent", function()
	if arg1 == "!OmniCC" then
		OmniCCMain:UnregisterEvent("ADDON_LOADED");
		LoadVariables();
	end
end);
OmniCCMain:RegisterEvent("ADDON_LOADED");