-- @ 프레임 확인
local function strchk(v)
	if type(v)=="string" then
		return getglobal(v)
	else
		return v
	end
	return nil
end

-------------------------------------------------------------------------------------■[ 프레임 ]
-- @ 위치설정
Tools_SetPoint = function (frame, p1, base, p2, x, y)
	p1 = ToolsData.PositionText[p1] or p1
	p2 = ToolsData.PositionText[p2] or p2
	if base == 0 then base = UIParent end
	strchk(frame):ClearAllPoints()
	strchk(frame):SetPoint(p1, base, p2, x, y)
end
-- @ 크기설정
Tools_SetSize = function (frame, width, height)
	strchk(frame):SetWidth(width)
	strchk(frame):SetHeight(height)
end
-- @ 크기가져오기
Tools_GetSize = function (frame)
	return strchk(frame):GetWidth(), strchk(frame):GetHeight()
end
-- @ 보더설정
Tools_SetBackdrop = function (frame, edgeSize, borderInset, strata, edgefile, bgfile)
	borderInset = borderInset or 4
	strata = strata or "BACKGROUND"
	edgefile = ToolsData.BorderTexture[edgefile] or ToolsData.BorderTexture[1]
	strchk(frame):SetFrameStrata(strata)
	strchk(frame):SetBackdrop({
		bgFile = bgfile or "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = edgefile,
		tile = true, tileSize = 5, edgeSize = edgeSize or 16,
		insets={left = borderInset,
			right = borderInset,
			top = borderInset,
			bottom = borderInset}})
	strchk(frame):SetBackdropBorderColor(1,1,1,1)
	strchk(frame):SetBackdropColor(0,0,0,0.4)
end
-- @ 색상 설정
Tools_SetBorderColor = function (frame, r, g, b, a, t)
	if t=="Border" then
		strchk(frame):SetBackdropBorderColor(r,g,b,a)
	else
		strchk(frame):SetBackdropColor(r,g,b,a)
	end
end
-- @ 보이기 설정
Tools_SetShow = function (frame, v)
	if (v == 1) or (v == true) then
		strchk(frame):Show()
	else
		strchk(frame):Hide()
	end
end
-- @ 일괄설정
FrameManager = function (v)
	if (v.Point) then
		Tools_SetPoint(v.Frame, unpack(v.Point))
	end
	if (v.Show) then
		Tools_SetShow(v.Frame, v.Show)
	end
	if (v.Size) then
		Tools_SetSize(v.Frame, unpack(v.Size))
	end
	if (v.Scale) then
		strchk(v.Frame):SetScale(v.Scale)
	end
	if (v.Border) then
		if (type(v.Border) == "table") then
			Tools_SetBackdrop(v.Frame, unpack(v.Border))
		elseif (v.Border == true) then
			Tools_SetBackdrop(v.Frame)
		end
	end
	if (v.Color) then
		if (v.Color.border) then
			strchk(v.Frame):SetBackdropBorderColor(unpack(v.Color.border))
		end
		if (v.Color.backdrop) then
			strchk(v.Frame):SetBackdropColor(unpack(v.Color.backdrop))
		end
	end
end
-------------------------------------------------------------------------------------■[ 미니맵 ]
-- @ 미니맵 저장 데이터 확인
local function checkMinimapTable()
	if not (ToolsMinimapData) then
		ToolsMinimapData = {	IconPos = {},
					Square = false,
					Lock = {}}
	end
end
-- 윈드 미니맵 설정
local function CheckMinimapAddons()
	if IsAddOnLoaded("WindMinimap") then
		if (WindMinimapBodyConfig[8]) then
			if WindMinimapBodyConfig[8] == 2 then
				ToolsMinimapData.Square = true
				return
			end
		end
	else	-- 사각 미니맵 애드온종류 추가시 대비
	end
	ToolsMinimapData.Square = false
end
-- @ 마우스 오버
local function MinimapOnEnterfunc(button, t)
	GameTooltip:SetOwner(button, "ANCHOR_TOPLEFT")
	GameTooltip:SetText(t)
end
-- @ 드래깅
local function MinimapOnDragfunc(v,s)
	ToolsMinimapData.Lock[v] = ToolsMinimapData.Lock[v] or false

	if ToolsMinimapData.Lock[v] == false then
		if (s == true) then
			getglobal(v):LockHighlight()
			getglobal(v.."Frame"):Show()
		elseif (s == false) then
			getglobal(v):UnlockHighlight()
			getglobal(v.."Frame"):Hide()
end	end	end
-- @ 위치설정
local function MinimapPosition(v, p)
	local x,y
	CheckMinimapAddons()
	if ToolsMinimapData.Square == true then
		x = math.max(-82, math.min(110 * cos(p) ,84))
		y = math.max(-86, math.min(110 * sin(p) ,82))
	else
		x = 80 * cos(p)
		y = 80 * sin(p)
	end
	
	Tools_SetPoint(v, 7, Minimap, 7, 52-x, y-52)
end
-- @ 업데이트
local function MinimapOnUpdatefunc(v)
	local xpos, ypos = GetCursorPosition()
	local xmin, ymin = Minimap:GetLeft(), Minimap:GetBottom()
	xpos = xmin - xpos / UIParent:GetScale() + 70
	ypos = ypos / UIParent:GetScale() - ymin - 70
	ToolsMinimapData.IconPos[v] = math.deg(math.atan2(ypos, xpos))
	if(ToolsMinimapData.IconPos[v] < 0) then
		ToolsMinimapData.IconPos[v] = ToolsMinimapData.IconPos[v] + 360
	end
	MinimapPosition(v, ToolsMinimapData.IconPos[v])
end
-- @ 미니맵 제작
CreateMinimapButton = function (v)
	checkMinimapTable()
	ToolsMinimapData.IconPos[v.name] = ToolsMinimapData.IconPos[v.name] or 0

	local button = CreateFrame("Button", v.name, Minimap, "ToolsMinimapButtonTemplate")
	CreateFrame("Frame", v.name.."Frame", getglobal(v.name)):Hide()

	getglobal(v.name.."Icon"):SetTexture(v.texture)

	button:SetScript("OnEnter", function () MinimapOnEnterfunc(button, v.tooltip) end)
	button:SetScript("OnLeave", function () GameTooltip:Hide() end)
	button:SetScript("OnDragStart", function () MinimapOnDragfunc(v.name, true) end)
	button:SetScript("OnDragStop", function () MinimapOnDragfunc(v.name, false) end)
	getglobal(button:GetName().."Frame"):SetScript("OnUpdate", function () MinimapOnUpdatefunc(v.name) end)

	if (v.script) then
		for script, func in pairs(v.script) do
			button:SetScript(script, func)
	end	end

	MinimapPosition(button, ToolsMinimapData.IconPos[v.name])
end
-------------------------------------------------------------------------------------■[ 이벤트 ]
-- @ 등록
ToolsEventReg = function (frame, e, func)
	local f = getglobal(frame) or CreateFrame("Frame", frame)
	if type(e) == "table" then
	    for i = 1, table.getn(e) do
		f:RegisterEvent(e[i])
	    end
	else
	    f:RegisterEvent(e)
	end
	f:SetScript("OnEvent", function() func(event) end)
end
-------------------------------------------------------------------------------------■[ 타이머 ]
local function print(v)
	ChatFrame1:AddMessage(v)
end
-- @ 테이블 설정
local tmpTimerData = {}

-- @ 타이머 프레임 제작/업데이트 스크립트 등록
CreateFrame("Frame", "ToolsCommonFrame"):SetScript("OnUpdate",
	function ()
		for k,v in pairs(tmpTimerData) do
			if (v.endtime <= GetTime()) then
				tmpTimerData[k] = nil
				if (v.func) then
					v.value = v.value or nil
					return v.func(v.value)
				else
					return true
	end	end	end	end)
-- @ 타이머 배치 실행
local function TimerBatch(head, v)
	local info, looptimes, loopvalues
	if ( type(v.time) == "table" ) then looptimes = v.time end
	if ( type(v.value) == "table" ) then loopvalues = v.value end
	if ( looptimes ) then
		local pretime = 0
		for i=1, table.getn(looptimes) do
			info = {}
			if ( loopvalues ) then
				info.value = loopvalues[i]
			else
				info.value = v.value
			end
			info.func = v.func or nil
			pretime = pretime + looptimes[i]
			info.time = pretime
			ToolsTimer(head..i, info)
		end
	elseif ( loopvalues ) then
		local looptime = v.time
		for i=1, table.getn(loopvalues) do
			info = {}
			info.time = looptime*i
			info.func = v.func or nil
			info.value = loopvalues[i]
			ToolsTimer(head..i, info)
		end
	end
end
-- @ 등록
ToolsTimer = function (head, v)
	if ( type(v.time) == "table" ) or ( type(v.value) == "table" ) then
		TimerBatch(head, v)
	else
		v.endtime = GetTime()+v.time
		tmpTimerData[head] = v
	end
end
-------------------------------------------------------------------------------------■[드롭다운]
-- @ 테이블 정의
ToolsDropdown = {}		-- 전체 테이블
ToolsDropdown.Common = {}	-- 공통 미니맵 테이블
ToolsDropdown.Common[1] = {}
ToolsDropdown.Common[2] = {}
ToolsDropdown.Common[3] = {}

local Dropdown;			-- 클릭시 선택할 테이블
-- @ 공통 미니맵 확인
local function CheckToolBox()
	if not (ToolsMinimap) then
		CreateMinimapButton({
			name = "ToolsMinimap",
			texture = "Interface\\Icons\\INV_Crate_01.blp",
			tooltip = "연장상자",
			script={OnClick = function ()
					ToolsDropdownOnClick("Common", ToolsMinimap)
				end}
		})
	end
end
-- @ 드롭다운 메뉴 레벨 설정
local function DropdownInit()
	if not ( ToolsDropdown[Dropdown][UIDROPDOWNMENU_MENU_LEVEL] ) then
		return
	end
	for i = 1, table.getn(ToolsDropdown[Dropdown][UIDROPDOWNMENU_MENU_LEVEL]) do
		local info = ToolsDropdown[Dropdown][UIDROPDOWNMENU_MENU_LEVEL][i]
		if (info.Check) then info.checked = info.checkedfunc(info.Check) end
		if (info.Text) then info.text= info.textfunc(info.Text) end
		if ( UIDROPDOWNMENU_MENU_LEVEL > 1 ) then
			if ( UIDROPDOWNMENU_MENU_VALUE == info.owner) then
				UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
			end
		else
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
		end
	end
	return
end
-- @ 연장상자에 등록
local leveltext = {"first","second","third"}

ToolsDropdownReg = function (v)
	CheckToolBox()
	for lv = 1, 3 do
		if (v[leveltext[lv]]) then
			for i = 1, table.getn(v[leveltext[lv]]) do
				table.insert(ToolsDropdown.Common[lv], v[leveltext[lv]][i])
			end
		end
	end
end
-- @ 연장상자에서 밸류값과 동일한 테이블 삭제 ( 변경되는 테이블용 )
ToolsDropdownRemove = function (value)
	for lv = 1, 3 do
		for i = 1, table.getn(ToolsDropdown.Common[lv]) do
			if ToolsDropdown.Common[lv][i].value == value then
				table.remove(ToolsDropdown.Common[lv], i)
			end
		end
	end
end
-- @ OnClick설정
ToolsDropdownOnClick = function (frame, relativeTo, point, x, y)
	if not getglobal(frame.."Dropdownframe") then
		CreateFrame("Frame", frame.."Dropdownframe", UIParent, "UIDropDownMenuTemplate")
		getglobal(frame.."Dropdownframe"):Hide()
	end
	if (Dropdown) then
		if (Dropdown == frame) and (DropDownList1:IsShown()) then
			getglobal(frame.."Dropdownframe"):Hide()
			CloseDropDownMenus()
			return
		else
			getglobal(Dropdown.."Dropdownframe"):Hide()
		end
	end
--	if (getglobal(frame.."Dropdownframe"):IsShown()) then
--		if (DropDownList1:IsShown()) then
--			getglobal(frame.."Dropdownframe"):Hide()
--			CloseDropDownMenus()
--			return
--		end
--	end
	Dropdown = frame
	local info = getglobal(frame.."Dropdownframe")
	info.relativeTo = relativeTo
	info.relativePoint = point or "BOTTOMLEFT"
	info.xOffset= x or 0
	info.yOffset= y or 0

	UIDropDownMenu_Initialize(info, DropdownInit, "MENU");
	ToggleDropDownMenu(1, nil, info)
	getglobal(frame.."Dropdownframe"):Show()
end